use 5.008004;
use ExtUtils::MakeMaker;
# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.


my $CONSTINCLUDE = 'headerfilegeneratedconstantwrappers.xsh';
my $MAINPACKAGENAME = 'Authen::SPNEGO::GSSAPI';

my @SEARCH_HEADER_DIR = map { ($_, $_ . '/gssapi') } qw(/usr/include /usr/include/local);

use Getopt::Long;
use vars qw($opt_gssapiimpl $opt_help);

&GetOptions("gssapiimpl=s", \$opt_gssapiimpl, "help", \$opt_help);


if ($opt_help){
print <<HELP;

==================================================================
BUILDING HELP
==================================================================

Hello - you obviously need Help! :-)

This additional command line options are available
for adjusting:

(1) --gssapiimpl /bla/local/heimdal
  This tells me where to look for your installed
  GSSAPI installation MIT or Heimdal.
  This must be the directory that contains
  the ./lib and ./include directories

HELP
;
  exit;
}


#-------------------------------------------------------------
print "\n Welcome to Achim GSSAPI setup \n";

my $ADDLINKPATH='';
my $ADDINCLUDEPATH='';

if ($opt_gssapiimpl) {
    print "\nTrying GSSAPI dir $opt_gssapiimpl...";
    checkdir ( $opt_gssapiimpl );
    #
    my $libdir = join '/', $opt_gssapiimpl, 'lib';
    checkdir( $libdir );
    #
    my $incdir = join '/', $opt_gssapiimpl, 'include';
    -e $incdir || fatal_error( "$incdir does not exist" );
    unless ( -e join '/', $incdir, 'gssapi.h' ) {
        $incdir = join '/', $incdir, 'gssapi';
    }
    checkdir( $incdir );
    $ADDLINKPATH="-L$libdir";
    $ADDINCLUDEPATH=$incdir;
    print "OK \n  using GSSAPI libs in $ADDLINKPATH \n  using GSSAPI includedir $ADDINCLUDEPATH\n";
} else {
    foreach ( @SEARCH_HEADER_DIR ) {
       print("\n  trying $_ ...");
       test_if_dir_available( $_ );
       my $fgssapih = join '/', $_, 'gssapi.h';
       if ( -e $fgssapih && -f $fgssapih && -r $fgssapih  ) {
          printf " found $fgssapih";
	  $ADDINCLUDEPATH = $_;
	  last;
       }
    }
    $ADDINCLUDEPATH || die "cannot find gssapi.h!";
}
#------------------------------------------------------
#------------------------------------------------------
my $FILEGSSAPI_H = join '/', $ADDINCLUDEPATH, 'gssapi.h';
-e $FILEGSSAPI_H || fatal_error( "$FILEGSSAPI_H does not exist" );
-f $FILEGSSAPI_H || fatal_error( "$FILEGSSAPI_H is dir, not plain file" );
-r $FILEGSSAPI_H || fatal_error( "$FILEGSSAPI_H is not readable" );
print "\n  using GSSAPI headers from $FILEGSSAPI_H \n";

my @GSSLIBS = map { "$ADDLINKPATH $_" } ( '-lgssapi_krb5', # MIT
                                          '-lgssapi -lkrb5 -lssl -lasn1 -lasn1 -lroken -lresolv -lcrypt -ldes -lcrypto' # Heimdal
                                         );

print "\n\n Linkconfiguration:\n", join "\n", @GSSLIBS, "\n";

my @GSS_S = cannibalize_header($FILEGSSAPI_H);
print "\n Importing constants ", join ' ', @GSS_S;
print "\n Writing $CONSTINCLUDE...";
open XSHFILE, ">$CONSTINCLUDE" or die $!;
print XSHFILE join "\n", map { gen_constwrapfunction('OM_uint32',
                            $_,
                            $MAINPACKAGENAME,
			    $MAINPACKAGENAME) } @GSS_S;
close XSHFILE;
print 'OK';

print "\n Writing GSSSExports...";
open PMFILE, '>lib/Authen/SPNEGO/GSSAPI/GSSSExports.pm' or die $!;
print PMFILE join "\n",
             "package $MAINPACKAGENAME" . '::GSSSExports;',
	     "use strict;",
	     "sub gss_s_constant_exports {",
	     'return qw(',  join ' ', @GSS_S  ,  ');',
	     "}",
	     "1;";
close  PMFILE;


#------------------------------------------------------
sub checkdir {
  my ($dir) = @_;
    -e $dir || fatal_error( "$dir does not exist" );
    -d $dir || fatal_error( "$dir not directory" );
    -r $dir || fatal_error( "$dir not readable" );
}
#------------------------------------------------------
sub fatal_error {
    print 'Fatal Error:', @_, "\n";
    exit;
}
#------------------------------------------------------



WriteMakefile(
    NAME              => 'Authen::SPNEGO::GSSAPI',
    VERSION_FROM      => 'lib/Authen/SPNEGO/GSSAPI.pm', # finds $VERSION
    #PREREQ_PM         => { 'MIME::Base64' => 3.01  }, # e.g., Module::Name => 1.1
    ($] >= 5.005 ?     ## Add these new keywords supported since 5.005
      (ABSTRACT_FROM  => 'lib/Authen/SPNEGO/GSSAPI.pm', # retrieve abstract from module
       AUTHOR         => 'Achim Grolms <achim@grolmsnet.de>') : ()),
       LIBS => \@GSSLIBS,
       INC  => $ADDINCLUDEPATH ? "-I$ADDINCLUDEPATH" : '',
       DEFINE            => '', # e.g., '-DHAVE_SOMETHING'

	# Un-comment this if you add C files to link with later:
    # OBJECT            => '$(O_FILES)', # link all the C files too
);



# ---------------------------------------------------------------------------
sub test_if_dir_available {
   my ($dir) = @_;
   return -e $dir && -d $dir && -r $dir;
}
# ---------------------------------------------------------------------------
sub cannibalize_header {
   my ($gssapiheader) = @_;
   $gssapiheader || die missing param '$gssapiheader';
   -e $gssapiheader || die "$gssapiheader does not exist!";
   my @r;
   open INGSSAPIH, $gssapiheader or die "error while opening $gssapiheader : $!";
   while(<INGSSAPIH>) {
       if (m/#define[ ]+(GSS_S_[A-Z_a-z]+)/) {
	  push @r, $1
       }
   }
   close INGSSAPIH or die $!;
   return @r;
}
# ---------------------------------------------------------------------------
sub gen_constwrapfunction {
   my ($returntyp, $constant, $modul, $package) = @_;
   return join "\n",,
	       "MODULE = $modul PACKAGE = $package",
	       "",
	       "$returntyp $constant()",
               "CODE:",
               "        RETVAL = $constant;",
               "OUTPUT:",
               "        RETVAL",
	       "\n";

}
# ---------------------------------------------------------------------------
