package Authen::SPNEGO::GSSAPI;

use 5.008004;
use strict;
use warnings;



require Exporter;

use Authen::SPNEGO::GSSAPI::GSSSExports;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Authen::SPNEGO::GSSAPI ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(gssapi_token_2_read_handle gss_buffer_desc_length hr_errormessage  GSS_C_NO_CONTEXT gss_context_time gss_display_status_GSS_CODE gss_display_status_MECH_CODE gss_import_name major_status_fields),
                                Authen::SPNEGO::GSSAPI::GSSSExports::gss_s_constant_exports() ] );

our @EXPORT_OK = (  @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);

our $VERSION = '0.02';

require XSLoader;
XSLoader::load('Authen::SPNEGO::GSSAPI', $VERSION);

#--------------------------------------------------------
sub hr_errormessage($$) {
    my ($major, $minor) = @_;
    my @r;

    push @r, sprintf('GSS_CODE (major): %x', $major) , gss_display_status_GSS_CODE( $major );
    push @r, "--";
    push @r, sprintf('MECH_CODE (minor): %x', $minor), gss_display_status_MECH_CODE( $minor );
    return @r;
}
#--------------------------------------------------------
sub gssapi_token_2_read_handle($) {
    my ($gssapitoken) = @_;


    pipe TOKENIN, TOKENOUT ;
    binmode TOKENOUT;
    binmode TOKENIN;

    my $tokenlength = gss_buffer_desc_length( $gssapitoken  );
    if ( $tokenlength ==  __dump_token2stream( *TOKENOUT, $gssapitoken) ) {
        close TOKENOUT or die $!;
	return *TOKENIN;
    } else {
        close TOKENOUT or die $!;
	close TOKENIN  or die $!;
	return undef;
    }

}
#--------------------------------------------------------

#--------------------------------------------------------

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Authen::SPNEGO::GSSAPI - Perl extension for using GSSAPI C-Bindings (RfC 1509)

=head1 SYNOPSIS


use MIME::Base64;
use Authen::SPNEGO::GSSAPI::SecurityContext;

eval {
   my $sec_context = Authen::SPNEGO::GSSAPI::SecurityContext->new(
                   { PRINCIPAL => 'HTTP/moerbsen.grolmsnet.lan' } );
   print "\n Time To live (TLL) of context: ",
          $sec_context->actual_ttl_secs();
   print "\n\n";
   *HAND = $sec_context->token_read_handle();

   my $temp = $/;
   local($/) = undef;  # slurp
   print encode_base64(<HAND>);
   close HAND;
};
if ($@) {
   print "\n Oh my good, runtimeerror!!!1111!\n\n", $@;
   exit;
}


See perldoc Authen::SPNEGO::GSSAPI::SecurityContext
for description of OO interface

=head1 DESCRIPTION

Authen::SPNEGO::GSSAPI - Perl extension for using GSSAPI RfC 1509 from
Implementations like MIT Kerberos or Heimdal for generation of
SPNEGO tokens.
Authen::SPNEGO::GSSAPI can be used as common GSSAPI bindings, too.

See perldoc  Authen::SPNEGO::GSSAPI::SecurityContext
for OO interface of the module

=head1 ERROR HANDLING

Most of the low-level GSSAPI-calls return two status
codes ($majorstate, $minorstate).
$majorstate is the GSS-Errorcode from general API,   $minorstate
is the MECH_CODE from underlying implematation.
if $majorstate = &GSS_S_COMPLETE everything ist fine (no errors),
in other case you can use hr_errormessage( $majorstate, $minorstate)
to get an array of humanreadable errormessages:


print join "\n",  hr_errormessage( $majorstate, $minorstate);



=head2 error handling methods

=over

=item hr_errormessage( $majorstate, $minorstate)

Returns an array of human-readable errormessages  converted from (  $majorstate, $minorstate )

=back

=head2 error handling methods (lower level)

=over

=item gss_display_status_GSS_CODE( $majorstate )

Returns an array of human-readable errormessages converted from the $majorstate errorcode

=item gss_display_status_MECH_CODE( $minorstate );

Returns an array of human-readable errormessages converted from the $minorstate errorcode

=item ($calling, $routine, $supll) = major_status_fields($majorstate);

Splits the $majorstate returncode to the 3 components $calling, $routine, $supll.
This is just a wrapper of RFC1509 Macros GSS_CALLING_ERROR(), GSS_ROUTINE_ERROR() and
GSS_SUPPLEMENTARY_INFO()

=back







=head1 EXPORTABLE CONSTANTS

The GSS_S_* constants described in RFC1509 are exported as functionwrappers.
Example: you can access GSS_S_COMPLETE as &GSS_S_COMPLETE from Perl.

=head1 LAZY METHODS

=over

=item gssapi_token_2_base64( $gssapi_token  );

Coverts binary $gssapi_token to Base64 representation

=back

=head1 LOW LEVEL METHODS

=over

=item ($majorstate, $minorstate, $gss_servername ) = gss_import_name( $servername );

returns the GSSAPI-internal representation of a servername,
if $majorstate == GSS_S_COMPLETE

=item ($majorstate, $minorstate, $secs) =  gss_context_time( $sec_context );

Returns on success the number of secnds the security handle $sec_context
will remain valid. If the context has already expired, zero will be returned.

=back





=head1 DEBUGGING

=over

=item Authen::SPNEGO::GSSAPI::enable_internal_debuggig();

Enables output of debug messages from the crappy internals from the module.
Useful to look for what bug crashes your application...
Disabeled by default.

=item  Authen::SPNEGO::GSSAPI::disable_internal_debuggig();

Disables output of debug messages from the crappy internals from the module.
Disabeled by default.

=back


=head1 SEE ALSO

perldoc  Authen::SPNEGO::GSSAPI::SecurityContext
for OO interface of the module

=head2 Standards

RfC 1509 describes the GSSAPI C-Bindings wrapped here



=head1 AUTHOR

Achim Grolms, E<lt>achim@grolmsnet.deE<gt>

many thanks to Harald Joerg who helped and contributed as usual :-)

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Achim Grolms

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.


=cut
